from __future__ import annotations

from biliass._core import get_danmaku_meta_size as get_danmaku_meta_size

from .biliass import (
    Danmaku2ASS as Danmaku2ASS,
    read_comments_bilibili_protobuf as read_comments_bilibili_protobuf,
    read_comments_bilibili_xml as read_comments_bilibili_xml,
)
